;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;     Here's the simulation routine:
#|
 We assume a constant (steady state) population of population-size animals,
		    with given masses determined by cicada masses of
		        (cicada-mean 1000)
		        (cicada-sd 200)
		    and
			(population-size 1000)
   Each female can heft up to heft times her mass (heft 2) 
   Each female can make up to hunt-attempts attempts to create a female
                        (hunt-attempts 10)  
   We start with an initial population of wasp masses at the beginning of the
   season, and allow them to hunt. Then we randomly select from the offspring
   another population-size of wasps for the next season, for n generations.
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun simulation-st-johns-annual-wasp-hunt-successes
  (
   n ;; how many years do you want? 
   &key 
   (heft 2.5)
   (population-size 100) 
   (hunt-attempts 10) 
   (alpha .5) ;; proportion of TG to use in future
   (initial-population-choice .5) ;; proportion of TG to use in initial
   (TB-mean 2761)
   (TB-sd 452)
   (TG-mean 1899)
   (TG-sd 322)
   lop ;; if you want to lop off some wasps, use the maximum value here
   (pre-title (format nil "Alpha: ~,3f;" alpha))
   (wasps 
    (let (
	  (choice1 (rand.binomial population-size 1 initial-population-choice))
	  (choice2 (rand.binomial population-size 1 initial-population-choice))
	  )
      (/
       (+ 
	(* choice1 (+ TG-mean (* TG-sd (rand.n population-size))))
	(* choice2 (+ TG-mean (* TG-sd (rand.n population-size))))
	(* (- 1 choice1) (+ TB-mean (* TB-sd (rand.n population-size))))
	(* (- 1 choice2) (+ TB-mean (* TB-sd (rand.n population-size))))
	)
       4
       )
      )
    )
   )
  "
 We assume a constant (steady state) population of n (population-size) animals,
		    with masses determined by cicada masses sampled from the 
                    Tibacin assumed normal (TG-mean and TG-sd, and TB-mean and
                    TB-sd). 
   Each female can heft up to heft (heft) times her mass.
   Each female can make up to hunt-attempts hunts to create a female
   We start with an initial population of wasp masses at the beginning of the
   season, and allow them to hunt. Then we randomly select from the offspring
   another population-size of wasps for the next season, for n generations.

;; Examples:
 (def simulation-st-johns-results 
   (simulation-st-johns-annual-wasp-hunt-successes 100
     :initial-population-choice .9
     :alpha .35
     :lop 800
     :heft 2.2
     )
   )
 (plot (first simulation-st-johns-results))
 (send my-plot :range 1 800 1200)
 (def simulation-st-johns-results 
   (simulation-st-johns-annual-wasp-hunt-successes 100
     :initial-population-choice .9
     :heft 2.2
     :lop 1000
     :alpha .35
     )
   )
 (plot (first simulation-st-johns-results))
 (send my-plot :range 1 800 1200)
 (tile (combine (transpose (split-list (select (agw) (iseq 5 0)) 3))) :height 300 :yshift 100)
 (def simulation-st-johns-results 
   (simulation-st-johns-annual-wasp-hunt-successes 100
     :initial-population-choice .9
     :heft 2.5
     :lop 1000
     :alpha .35
     )
   )
 (plot (first simulation-st-johns-results))
 (send my-plot :range 1 800 1200)
 (tile (combine (transpose (split-list (select (agw) (iseq 8 0)) 3))) :height 200 :yshift 100)
;; (tile 
;;  (select (agw) (iseq 2 0)) :height 300 :width 300 :yshift 100 :nrow 1 :ncol 3)
"
  (let (
	population-means
	population-sds
	(cutpoints (/
		    (rseq 
		     (- TG-mean (* TG-sd 3))
		     (+ TB-mean (* TB-sd 3))
		     20)
		    2)
		   )
	(cutpoints (rseq 500 2000 30))
	)
    (setq initialpop wasps)
    (if lop (setq wasps (sample (select wasps (where wasps lop #'<))
				population-size t)))
    (setq trimmedpop wasps)
    (print (list 'wasps (min wasps) (max wasps)))
    (title (strcat pre-title "Initial Wasp Population"))
    (histo wasps :my-cutpoints cutpoints)
    (dotimes (runs n)
      (let (offspring-masses)
	(dotimes
	    (i population-size)
	  (let ((wasp (elt wasps i)))
	    (dotimes (j hunt-attempts)
	      (let* (
		     (choice (rand.binomial 2 1 alpha))
		     (tmp  
		      (+
		       (* choice 
			  (+ TG-mean (* TG-sd (rand.n 2))))
		       (* (- 1 choice)
			  (+ TB-mean (* TB-sd (rand.n 2))))
		       )
		      )
		     )
		(if (and
		     (> (* heft wasp) (elt tmp 0))
		     (> (* heft wasp) (elt tmp 1))
		     )
		    (setq offspring-masses (append offspring-masses (list (sum tmp))))
		  ;; Otherwise she might produce a male, or get nothing.
		  )
		)
	      )
	    )
	  )
	(setq wasps (/ 
		     (sample offspring-masses population-size t)
		     4)
	      wasp-mean (mean wasps)
	      wasp-sd (sd wasps)
	      population-means (append population-means (list wasp-mean))
	      population-sds (append population-sds (list wasp-sd))
	      )
	(format t 
		"~%Iteration ~d: ~d wasps: mean ~,3f; sd ~,3f; offspring length: ~d" 
		runs 
		(length wasps) 
		wasp-mean
		wasp-sd
		(length offspring-masses)
		)
	))

    (title (strcat pre-title "Final Wasp Population"))
    (histo wasps :my-cutpoints cutpoints)
    (setq finalpop wasps)

    (format t "~% Newberry Female mean:  613; sd: 143")
    (format t "~%St. Johns Female mean: 1134; sd: 319")
    (format t "~%Grand mean: ~,3f~%Grand sd: ~,3f~%"
	    (mean population-means)
	    (let ((tmp (mean (square population-sds))))
	      (sqrt
	       (+ 
		tmp
		(/ (* (- n 1) (- (variance population-means) tmp))
		   (- (* n population-size) 1))))
	      )
	    )
    (list population-means population-sds)
    )
  )


(let ()
  (def simulation-st-johns-results 
       (simulation-st-johns-annual-wasp-hunt-successes 
	200
	:initial-population-choice .2
	:population-size 1000
	:heft 2.2
	:lop 1000
	:alpha .35
	)
       )
  (plot (first simulation-st-johns-results))
  (send my-plot :range 1 800 1200)
  (write-matrix-to-file 
   (bind-rows
    (list "meanPops" "sdPops")
    (bind-columns 
     (first simulation-st-johns-results)
     (second simulation-st-johns-results)
     )
    )
   "grayson_simulation_data_stats.txt"
   )
  (write-matrix-to-file 
   (bind-rows
    (list "initialPop" "trimmedPop" "finalPop")
    (bind-columns 
     initialpop
     trimmedpop
     finalpop
     )
    )
   "grayson_simulation_data_pops.txt"
   )

  (def simulation-st-johns-results 
       (simulation-st-johns-annual-wasp-hunt-successes 
	200
	:initial-population-choice .2
	:population-size 1000
	:heft 2.2
	:lop 1000
	:alpha .35
	:wasps initialpop
	)
       )
  (plot (first simulation-st-johns-results))
  (send my-plot :range 1 800 1200)
  (write-matrix-to-file 
   (bind-rows
    (list "meanPops" "sdPops")
    (bind-columns 
     (first simulation-st-johns-results)
     (second simulation-st-johns-results)
     )
    )
   "grayson_simulation_data_stats_sag.txt"
   )
  (write-matrix-to-file 
   (bind-rows
    (list "initialPop" "trimmedPop" "finalPop")
    (bind-columns 
     initialpop
     trimmedpop
     finalpop
     )
    )
   "grayson_simulation_data_pops_sag.txt"
   )
  )

(let ()
  (setq
   a_pops
   (read-matrix-data-with-one-line-header "grayson_simulation_data_pops_sag.txt")
   a_stats
   (read-matrix-data-with-one-line-header "grayson_simulation_data_stats_sag.txt")
   
   initialpop_sag      initialpop
   trimmedpop_sag     trimmedpop
   finalpop_sag     finalpop
   b_pops
   (read-matrix-data-with-one-line-header "grayson_simulation_data_pops.txt")
   b_stats
   (read-matrix-data-with-one-line-header "grayson_simulation_data_stats.txt")
   )
  (histo initialpop_sag)
  (histo initialpop)
  (histo finalpop_sag :my-cutpoints (rseq 500 2000 31))
  (histo finalpop :my-cutpoints (rseq 500 2000 31))
  )
(sd finalpop)
(mean finalpop)
(sd finalpop_sag)
(mean finalpop_sag)
